@@ -23,10 +23,11 @@ import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.io.Reader;
-import java.io.Writer;
 import java.nio.file.Files;
-import java.util.Enumeration;
+import java.nio.file.Path;
+import java.nio.file.StandardCopyOption;
+import java.util.Objects;
+import java.util.function.Predicate;
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
 import java.util.regex.Pattern;
@@ -42,11 +43,7 @@ import org.apache.maven.plugin.MojoFailureException;
 import org.apache.maven.plugins.annotations.Component;
 import org.apache.maven.plugins.annotations.Mojo;
 import org.apache.maven.plugins.annotations.Parameter;
-import org.codehaus.plexus.util.FileUtils;
-import org.codehaus.plexus.util.IOUtil;
 import org.codehaus.plexus.util.StringUtils;
-import org.codehaus.plexus.util.xml.XmlStreamReader;
-import org.codehaus.plexus.util.xml.XmlStreamWriter;
 import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
 import org.eclipse.aether.DefaultRepositoryCache;
 import org.eclipse.aether.DefaultRepositorySystemSession;
@@ -60,6 +57,10 @@ import org.eclipse.aether.installation.InstallationException;
 import org.eclipse.aether.repository.LocalRepository;
 import org.eclipse.aether.repository.LocalRepositoryManager;
 import org.eclipse.aether.util.artifact.SubArtifact;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import static java.util.Objects.isNull;
 
 /**
  * Installs a file in the local repository.
@@ -68,7 +69,8 @@ import org.eclipse.aether.util.artifact.SubArtifact;
  */
 @Mojo(name = "install-file", requiresProject = false, aggregator = true, threadSafe = true)
 public class InstallFileMojo extends AbstractMojo {
-    private static final String LS = System.getProperty("line.separator");
+    private static final String LS = System.lineSeparator();
+    private final Logger log = LoggerFactory.getLogger(getClass());
 
     @Component
     private RepositorySystem repositorySystem;
@@ -113,6 +115,15 @@ public class InstallFileMojo extends AbstractMojo {
     @Parameter(property = "classifier")
     private String classifier;
 
+    /**
+     * Extension of the artifact to be installed. If set, will override plugin own logic to detect extension. If not set,
+     * as Maven expected, packaging determines the artifact extension.
+     *
+     * @since 3.1.3
+     */
+    @Parameter(property = "extension")
+    private String extension;
+
     /**
      * The file to be installed in the local repository.
      */
@@ -162,11 +173,15 @@ public class InstallFileMojo extends AbstractMojo {
     @Parameter(property = "localRepositoryPath")
     private File localRepositoryPath;
 
+    private static final Predicate<String> IS_EMPTY = s -> isNull(s) || s.isEmpty();
+
+    private static final Predicate<String> IS_POM_PACKAGING = "pom"::equals;
+
     @Override
     public void execute() throws MojoExecutionException, MojoFailureException {
         if (!file.exists()) {
             String message = "The specified file '" + file.getPath() + "' does not exist";
-            getLog().error(message);
+            log.error(message);
             throw new MojoFailureException(message);
         }
 
@@ -186,23 +201,23 @@ public class InstallFileMojo extends AbstractMojo {
                     newSession, new LocalRepository(localRepositoryPath, contentType));
             newSession.setLocalRepositoryManager(localRepositoryManager);
             repositorySystemSession = newSession;
-            getLog().debug("localRepoPath: "
-                    + localRepositoryManager.getRepository().getBasedir());
+            log.debug(
+                    "localRepoPath: {}", localRepositoryManager.getRepository().getBasedir());
         }
 
         File temporaryPom = null;
 
-        if (pomFile != null) {
-            processModel(readModel(pomFile));
-        } else {
+        if (pomFile == null) {
             temporaryPom = readingPomFromJarFile();
             if (!Boolean.TRUE.equals(generatePom)) {
                 pomFile = temporaryPom;
-                getLog().debug("Using JAR embedded POM as pomFile");
+                log.debug("Using JAR embedded POM as pomFile");
             }
+        } else {
+            processModel(readModel(pomFile));
         }
 
-        if (groupId == null || artifactId == null || version == null || packaging == null) {
+        if (isNull(groupId) || isNull(artifactId) || isNull(version) || isNull(packaging)) {
             throw new MojoExecutionException("The artifact information is incomplete: 'groupId', 'artifactId', "
                     + "'version' and 'packaging' are required.");
         }
@@ -213,18 +228,29 @@ public class InstallFileMojo extends AbstractMojo {
 
         InstallRequest installRequest = new InstallRequest();
 
-        boolean isFilePom = classifier == null && "pom".equals(packaging);
-        if (!isFilePom) {
+        String mainArtifactExtension;
+        if (classifier == null && "pom".equals(packaging)) {
+            mainArtifactExtension = "pom";
+        } else {
             ArtifactType artifactType =
-                    repositorySystemSession.getArtifactTypeRegistry().get(packaging);
-            if (artifactType != null
-                    && StringUtils.isEmpty(classifier)
-                    && !StringUtils.isEmpty(artifactType.getClassifier())) {
-                classifier = artifactType.getClassifier();
+                    session.getRepositorySession().getArtifactTypeRegistry().get(packaging);
+            if (artifactType != null) {
+                if (StringUtils.isEmpty(classifier) && !StringUtils.isEmpty(artifactType.getClassifier())) {
+                    classifier = artifactType.getClassifier();
+                }
+                mainArtifactExtension = artifactType.getExtension();
+            } else {
+                mainArtifactExtension = packaging;
             }
         }
+        if (extension != null && !Objects.equals(extension, mainArtifactExtension)) {
+            log.warn(
+                    "Main artifact extension should be '{}' but was overridden to '{}'",
+                    mainArtifactExtension,
+                    extension);
+        }
         Artifact mainArtifact = new DefaultArtifact(
-                        groupId, artifactId, classifier, isFilePom ? "pom" : getExtension(file), version)
+                        groupId, artifactId, classifier, extension != null ? extension : mainArtifactExtension, version)
                 .setFile(file);
         installRequest.addArtifact(mainArtifact);
 
@@ -236,17 +262,17 @@ public class InstallFileMojo extends AbstractMojo {
                     + LS + LS + "File in question is: " + file + LS);
         }
 
-        if (!"pom".equals(packaging)) {
-            if (pomFile != null) {
-                installRequest.addArtifact(new SubArtifact(mainArtifact, "", "pom", pomFile));
-            } else {
+        if (!IS_POM_PACKAGING.test(packaging)) {
+            if (isNull(pomFile)) {
                 if (Boolean.TRUE.equals(generatePom) || (generatePom == null && !pomLocalFile.exists())) {
                     temporaryPom = generatePomFile();
-                    getLog().debug("Installing generated POM");
+                    log.debug("Installing generated POM");
                     installRequest.addArtifact(new SubArtifact(mainArtifact, "", "pom", temporaryPom));
                 } else if (generatePom == null) {
-                    getLog().debug("Skipping installation of generated POM, already present in local repository");
+                    log.debug("Skipping installation of generated POM, already present in local repository");
                 }
+            } else {
+                installRequest.addArtifact(new SubArtifact(mainArtifact, "", "pom", pomFile));
             }
         }
 
@@ -270,70 +296,40 @@ public class InstallFileMojo extends AbstractMojo {
         }
     }
 
+    private static final Pattern POM_ENTRY_PATTERN = Pattern.compile("META-INF/maven/.*/pom\\.xml");
+
+    private static final Predicate<JarEntry> IS_POM_ENTRY =
+            entry -> POM_ENTRY_PATTERN.matcher(entry.getName()).matches();
+
     private File readingPomFromJarFile() throws MojoExecutionException {
-        File pomFile = null;
 
-        JarFile jarFile = null;
-        try {
-            Pattern pomEntry = Pattern.compile("META-INF/maven/.*/pom\\.xml");
+        String base = file.getName();
+        if (base.contains(".")) {
+            base = base.substring(0, base.lastIndexOf('.'));
+        }
 
-            jarFile = new JarFile(file);
+        try (JarFile jarFile = new JarFile(file)) {
 
-            Enumeration<JarEntry> jarEntries = jarFile.entries();
+            JarEntry pomEntry = jarFile.stream().filter(IS_POM_ENTRY).findAny().orElse(null);
 
-            while (jarEntries.hasMoreElements()) {
-                JarEntry entry = jarEntries.nextElement();
-
-                if (pomEntry.matcher(entry.getName()).matches()) {
-                    getLog().debug("Loading " + entry.getName());
-
-                    InputStream pomInputStream = null;
-                    OutputStream pomOutputStream = null;
-
-                    try {
-                        pomInputStream = jarFile.getInputStream(entry);
-
-                        String base = file.getName();
-                        if (base.indexOf('.') > 0) {
-                            base = base.substring(0, base.lastIndexOf('.'));
-                        }
-                        pomFile = File.createTempFile(base, ".pom");
-
-                        pomOutputStream = Files.newOutputStream(pomFile.toPath());
-
-                        IOUtil.copy(pomInputStream, pomOutputStream);
-
-                        pomOutputStream.close();
-                        pomOutputStream = null;
-
-                        pomInputStream.close();
-                        pomInputStream = null;
-
-                        processModel(readModel(pomFile));
-
-                        break;
-                    } finally {
-                        IOUtil.close(pomInputStream);
-                        IOUtil.close(pomOutputStream);
-                    }
-                }
+            if (isNull(pomEntry)) {
+                // This means there is no entry which matches the "pom.xml"...(or in other words: not packaged by Maven)
+                log.info("pom.xml not found in {}", file.getName());
+                return null;
             }
 
-            if (pomFile == null) {
-                getLog().info("pom.xml not found in " + file.getName());
-            }
+            Path tempPomFile = Files.createTempFile(base, ".pom");
+
+            Files.copy(jarFile.getInputStream(pomEntry), tempPomFile, StandardCopyOption.REPLACE_EXISTING);
+
+            log.debug("Loading {}", pomEntry.getName());
+            processModel(readModel(tempPomFile.toFile()));
+            return tempPomFile.toFile();
+
         } catch (IOException e) {
             // ignore, artifact not packaged by Maven
-        } finally {
-            if (jarFile != null) {
-                try {
-                    jarFile.close();
-                } catch (IOException e) {
-                    // we did our best
-                }
-            }
+            return null;
         }
-        return pomFile;
     }
 
     /**
@@ -344,21 +340,14 @@ public class InstallFileMojo extends AbstractMojo {
      * @throws MojoExecutionException If the POM could not be parsed.
      */
     private Model readModel(File pomFile) throws MojoExecutionException {
-        Reader reader = null;
-        try {
-            reader = new XmlStreamReader(pomFile);
-            final Model model = new MavenXpp3Reader().read(reader);
-            reader.close();
-            reader = null;
-            return model;
+        try (InputStream reader = Files.newInputStream(pomFile.toPath())) {
+            return new MavenXpp3Reader().read(reader);
         } catch (FileNotFoundException e) {
             throw new MojoExecutionException("File not found " + pomFile, e);
         } catch (IOException e) {
             throw new MojoExecutionException("Error reading POM " + pomFile, e);
         } catch (XmlPullParserException e) {
             throw new MojoExecutionException("Error parsing POM " + pomFile, e);
-        } finally {
-            IOUtil.close(reader);
         }
     }
 
@@ -419,21 +408,15 @@ public class InstallFileMojo extends AbstractMojo {
      */
     private File generatePomFile() throws MojoExecutionException {
         Model model = generateModel();
-
-        Writer writer = null;
         try {
-            File pomFile = File.createTempFile("mvninstall", ".pom");
+            Path tempPomFile = Files.createTempFile("mvninstall", ".pom");
 
-            writer = new XmlStreamWriter(pomFile);
-            new MavenXpp3Writer().write(writer, model);
-            writer.close();
-            writer = null;
-
-            return pomFile;
+            try (OutputStream writer = Files.newOutputStream(tempPomFile)) {
+                new MavenXpp3Writer().write(writer, model);
+                return tempPomFile.toFile();
+            }
         } catch (IOException e) {
             throw new MojoExecutionException("Error writing temporary POM file: " + e.getMessage(), e);
-        } finally {
-            IOUtil.close(writer);
         }
     }
 
@@ -458,18 +441,6 @@ public class InstallFileMojo extends AbstractMojo {
 
     // these below should be shared (duplicated in m-install-p, m-deploy-p)
 
-    /**
-     * Specialization of {@link FileUtils#getExtension(String)} that honors various {@code tar.xxx} combinations.
-     */
-    private String getExtension(final File file) {
-        String filename = file.getName();
-        if (filename.contains(".tar.")) {
-            return "tar." + FileUtils.getExtension(filename);
-        } else {
-            return FileUtils.getExtension(filename);
-        }
-    }
-
     /**
      * Returns {@code true} if passed in string is "valid Maven ID" (groupId or artifactId).
      */
