@@ -19,9 +19,10 @@
 package org.apache.maven.plugins.install;
 
 import java.io.File;
-import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
+import java.util.function.Predicate;
+import java.util.stream.Collectors;
 
 import org.apache.maven.RepositoryUtils;
 import org.apache.maven.execution.MavenSession;
@@ -37,8 +38,10 @@ import org.apache.maven.plugins.annotations.Parameter;
 import org.apache.maven.project.MavenProject;
 import org.apache.maven.project.artifact.ProjectArtifact;
 import org.eclipse.aether.RepositorySystem;
+import org.eclipse.aether.artifact.Artifact;
 import org.eclipse.aether.installation.InstallRequest;
 import org.eclipse.aether.installation.InstallationException;
+import org.eclipse.aether.util.artifact.ArtifactIdUtils;
 
 /**
  * Installs the project's main artifact, and any other artifacts attached by other plugins in the lifecycle, to the
@@ -57,9 +60,6 @@ public class InstallMojo extends AbstractMojo {
     @Parameter(defaultValue = "${project}", readonly = true, required = true)
     private MavenProject project;
 
-    @Parameter(defaultValue = "${reactorProjects}", required = true, readonly = true)
-    private List<MavenProject> reactorProjects;
-
     @Parameter(defaultValue = "${plugin}", required = true, readonly = true)
     private PluginDescriptor pluginDescriptor;
 
@@ -122,15 +122,15 @@ public class InstallMojo extends AbstractMojo {
             getLog().info("Skipping artifact installation");
             putState(State.SKIPPED);
         } else {
-            if (!installAtEnd) {
+            if (installAtEnd) {
+                getLog().info("Deferring install for " + project.getGroupId() + ":" + project.getArtifactId() + ":"
+                        + project.getVersion() + " at end");
+                putState(State.TO_BE_INSTALLED);
+            } else {
                 InstallRequest request = new InstallRequest();
                 processProject(project, request);
                 installProject(request);
                 putState(State.INSTALLED);
-            } else {
-                getLog().info("Deferring install for " + project.getGroupId() + ":" + project.getArtifactId() + ":"
-                        + project.getVersion() + " at end");
-                putState(State.TO_BE_INSTALLED);
             }
         }
 
@@ -149,35 +149,27 @@ public class InstallMojo extends AbstractMojo {
     }
 
     private boolean allProjectsMarked(List<MavenProject> allProjectsUsingPlugin) {
-        for (MavenProject reactorProject : allProjectsUsingPlugin) {
-            if (!hasState(reactorProject)) {
-                return false;
-            }
-        }
-        return true;
+        return allProjectsUsingPlugin.stream().allMatch(this::hasState);
     }
 
+    private final Predicate<MavenProject> hasMavenInstallPluginExecution =
+            rp -> hasExecution(rp.getPlugin("org.apache.maven.plugins:maven-install-plugin"));
+
     private List<MavenProject> getAllProjectsUsingPlugin() {
-        ArrayList<MavenProject> result = new ArrayList<>();
-        for (MavenProject reactorProject : reactorProjects) {
-            if (hasExecution(reactorProject.getPlugin("org.apache.maven.plugins:maven-install-plugin"))) {
-                result.add(reactorProject);
-            }
-        }
-        return result;
+        return session.getProjects().stream()
+                .filter(hasMavenInstallPluginExecution)
+                .collect(Collectors.toList());
     }
 
+    private final Predicate<PluginExecution> havingGoals = pe -> !pe.getGoals().isEmpty();
+    private final Predicate<PluginExecution> nonePhase = pe -> !"none".equalsIgnoreCase(pe.getPhase());
+
     private boolean hasExecution(Plugin plugin) {
         if (plugin == null) {
             return false;
         }
 
-        for (PluginExecution execution : plugin.getExecutions()) {
-            if (!execution.getGoals().isEmpty() && !"none".equalsIgnoreCase(execution.getPhase())) {
-                return true;
-            }
-        }
-        return false;
+        return plugin.getExecutions().stream().filter(havingGoals).anyMatch(nonePhase);
     }
 
     private void installProject(InstallRequest request) throws MojoExecutionException {
@@ -194,31 +186,52 @@ public class InstallMojo extends AbstractMojo {
      * @throws MojoExecutionException if project is badly set up.
      */
     private void processProject(MavenProject project, InstallRequest request) throws MojoExecutionException {
-        if (isFile(project.getFile())) {
-            request.addArtifact(RepositoryUtils.toArtifact(new ProjectArtifact(project)));
-        } else {
-            throw new MojoExecutionException("The project POM could not be attached");
+        // always exists, as project exists
+        Artifact pomArtifact = RepositoryUtils.toArtifact(new ProjectArtifact(project));
+        // always exists, but at "init" is w/o file (packaging plugin assigns file to this when packaged)
+        Artifact projectArtifact = RepositoryUtils.toArtifact(project.getArtifact());
+
+        // pom project: pomArtifact and projectArtifact are SAME
+        // jar project: pomArtifact and projectArtifact are DIFFERENT
+        // incomplete project: is not pom project and projectArtifact has no file
+
+        // we must compare coordinates ONLY (as projectArtifact may not have file, and Artifact.equals factors it in)
+        // BUT if projectArtifact has file set, use that one
+        if (ArtifactIdUtils.equalsId(pomArtifact, projectArtifact)) {
+            if (isFile(projectArtifact.getFile())) {
+                pomArtifact = projectArtifact;
+            }
+            projectArtifact = null;
         }
 
-        if (!"pom".equals(project.getPackaging())) {
-            org.apache.maven.artifact.Artifact mavenMainArtifact = project.getArtifact();
-            if (isFile(mavenMainArtifact.getFile())) {
-                request.addArtifact(RepositoryUtils.toArtifact(mavenMainArtifact));
+        if (isFile(pomArtifact.getFile())) {
+            request.addArtifact(pomArtifact);
+        } else {
+            throw new MojoExecutionException(
+                    "The POM for project " + project.getArtifactId() + " could not be attached");
+        }
+
+        // is not packaged, is "incomplete"
+        boolean isIncomplete = projectArtifact != null && !isFile(projectArtifact.getFile());
+        if (projectArtifact != null) {
+            if (!isIncomplete) {
+                request.addArtifact(projectArtifact);
             } else if (!project.getAttachedArtifacts().isEmpty()) {
                 if (allowIncompleteProjects) {
                     getLog().warn("");
-                    getLog().warn("The packaging plugin for this project did not assign");
+                    getLog().warn("The packaging plugin for project " + project.getArtifactId() + " did not assign");
                     getLog().warn("a main file to the project but it has attachments. Change packaging to 'pom'.");
                     getLog().warn("");
                     getLog().warn("Incomplete projects like this will fail in future Maven versions!");
                     getLog().warn("");
                 } else {
-                    throw new MojoExecutionException("The packaging plugin for this project did not assign "
-                            + "a main file to the project but it has attachments. Change packaging to 'pom'.");
+                    throw new MojoExecutionException("The packaging plugin for project " + project.getArtifactId()
+                            + " did not assign a main file to the project but it has attachments. Change packaging"
+                            + " to 'pom'.");
                 }
             } else {
-                throw new MojoExecutionException(
-                        "The packaging for this project did not assign a file to the build artifact");
+                throw new MojoExecutionException("The packaging plugin for project " + project.getArtifactId()
+                        + " did not assign a file to the build artifact");
             }
         }
 
