@@ -1,7 +1,3 @@
-package org.apache.maven.plugins.install.stubs;
-
-import java.io.File;
-
 /*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -11,7 +7,7 @@ import java.io.File;
  * "License"); you may not use this file except in compliance
  * with the License.  You may obtain a copy of the License at
  *
- *  http://www.apache.org/licenses/LICENSE-2.0
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing,
  * software distributed under the License is distributed on an
@@ -20,19 +16,19 @@ import java.io.File;
  * specific language governing permissions and limitations
  * under the License.
  */
+package org.apache.maven.plugins.install.stubs;
 
-public class AttachedArtifactStub0
-    extends InstallArtifactStub
-{
-    public String getArtifactId()
-    {
+import java.io.File;
+
+public class AttachedArtifactStub0 extends InstallArtifactStub {
+    public String getArtifactId() {
         return "attached-artifact-test-0";
     }
 
-    public File getFile()
-    {
-        return new File( System.getProperty( "basedir" ),
-                         "target/test-classes/unit/basic-install-test-with-attached-artifacts/"
-                             + "target/maven-install-test-1.0-SNAPSHOT.jar" );
+    public File getFile() {
+        return new File(
+                System.getProperty("basedir"),
+                "target/test-classes/unit/basic-install-test-with-attached-artifacts/"
+                        + "target/maven-install-test-1.0-SNAPSHOT.jar");
     }
 }
